﻿Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.Data.SqlTypes
Imports System.Text
Imports Microsoft.SqlServer.Server

<Serializable()> _
<Microsoft.SqlServer.Server.SqlUserDefinedType(Format.Native, ValidationMethodName:="ValidatePoint")> _
Public Structure Point
    Implements INullable

#Region "Private Members"

    Private m_Null As Boolean
    Private _x As Double
    Private _y As Double

#End Region

#Region "Public Properties"

    Public Property X() As Double
        Get
            Return (Me._x)
        End Get

        Set(ByVal Value As Double)
            Dim temp As Double = _x
            _x = Value
            If Not ValidatePoint() Then
                _x = temp
                Throw New ArgumentException("Invalid X coordinate value.")
            End If
        End Set
    End Property

    Public Property Y() As Double
        Get
            Return (Me._y)
        End Get

        Set(ByVal Value As Double)
            Dim temp As Double = _y
            _y = Value
            If Not ValidatePoint() Then
                _y = temp
                Throw New ArgumentException("Invalid Y coordinate value.")
            End If
        End Set
    End Property

#End Region

#Region "Null Handling"

    Public ReadOnly Property IsNull() As Boolean Implements INullable.IsNull
        Get
            Return m_Null
        End Get
    End Property

    Public Shared ReadOnly Property Null() As Point
        Get
            Dim objPoint As Point = New Point
            objPoint.m_Null = True
            Return objPoint
        End Get
    End Property

#End Region

#Region "Private Methods"

    ''' <summary>
    ''' Kleines Helferlein zum Prüfen der Werte auf Gültigkeit
    ''' </summary>
    ''' <returns>True, wenn gültig, False wenn nicht</returns>

    Private Function ValidatePoint() As Boolean
        ' Beider Koordinaten sollen größer oder gleich 0 sein.
        If (_x >= 0) And (_y >= 0) Then
            Return True
        Else
            Return False
        End If
    End Function

#End Region

#Region "Public Methods"

    ''' <summary>
    ''' Darstellung des Datentyps als String
    ''' </summary>
    ''' <returns>Der String</returns>

    Public Overrides Function ToString() As String
        Dim sReturn As String = "NULL"
        If Not Me.IsNull Then
            Dim builder As StringBuilder = New StringBuilder()
            builder.Append(_x)
            builder.Append("|")
            builder.Append(_y)
            sReturn = builder.ToString()
        End If
        Return sReturn
    End Function

    ''' <summary>
    ''' Umwandlung eines Strings in den Datentyp
    ''' </summary>
    ''' <param name="s">Der String Wert</param>
    ''' <returns>Der Punkt</returns>

    <SqlMethod(OnNullCall:=False)> _
    Public Shared Function Parse(ByVal s As SqlString) As Point

        Dim objPoint As New Point()

        ' wenn nix dann gibbet auch nix zurück
        If s.IsNull Then
            objPoint = Null
        End If

        ' Verarbeiten des Eingabestrings zum bestimmen der Koordinaten
        ' da kann noch jede Menge "Validation" Code rein
        Dim arrCoordinates() As String = s.Value.Split("|"c)
        objPoint.X = Int32.Parse(arrCoordinates(0))
        objPoint.Y = Int32.Parse(arrCoordinates(1))

        ' Ergebnis validieren
        If Not objPoint.ValidatePoint() Then
            Throw New ArgumentException("Invalid XY coordinate values.")
        End If

        ' Rückgabe des Punktes
        Return objPoint

    End Function

#End Region

#Region "Erweiterung"

    ''' <summary>
    ''' Setzen aller Eigenschaften des Punktes
    ''' </summary>
    ''' <param name="x"></param>
    ''' <param name="y"></param>

    <SqlMethod(OnNullCall:=False, IsMutator:=True)> _
    Public Sub SetValue(ByVal xParam As Double, ByVal yParam As Double)

        Me.X = xParam
        Me.Y = yParam

    End Sub

#End Region

End Structure

